@@ -2,11 +2,9 @@ from . import autosimilarity_computation
 from . import barwise_input
 from . import data_manipulation
 from . import CBM_algorithm
-from . import foote_novelty
+#from . import foote_novelty
 
 from .model import current_plot
 from .model import errors
-from .model import features
-
-from .scripts import overall_scripts
-from .scripts import default_path
\ No newline at end of file
+from .model import signal_to_spectrogram
+from .model import display_results
