@@ -12,9 +12,11 @@ import as_seg.model.errors as err
 
 import numpy as np
 import madmom.features.downbeats as dbt
-import soundfile as sf
+import madmom.features.beats as bt
 import mir_eval
 import scipy
+import librosa
+import math
 
 # %% Read and treat inputs
 def get_bars_from_audio(song_path):
@@ -47,23 +49,49 @@ def get_bars_from_audio(song_path):
     act = dbt.RNNDownBeatProcessor()(song_path)
     proc = dbt.DBNDownBeatTrackingProcessor(beats_per_bar=[3,4], fps=100)
     song_beats = proc(act)
-    downbeats_times = []
+    downbeats_times = [song_beats[0][0]]
     
-    if song_beats[0][1] != 1: # Adding a first downbeat at the start of the song
-        downbeats_times.append(0.1)
-    for beat in song_beats:
+    for beat in song_beats[1:]: # The first beat is already added
         if beat[1] == 1: # If the beat is a downbeat
             downbeats_times.append(beat[0])
             
-    # The following block of code artificially adds bars to the end of the song, in order to span the total song length.
-    # It seems like a good idea initially but may be detrimental, and should be debated anyway.
-    average_bar_length = np.mean([downbeats_times[i + 1] - downbeats_times[i] for i in range(len(downbeats_times) - 1)]) # average bar length in the song
     song_length = act.shape[0]/100 # Total length of the song
-    while downbeats_times[-1] + 1.1*average_bar_length < song_length: # As long as the bar estimation does not cover the entire song
-        downbeats_times.append(round(downbeats_times[-1] + average_bar_length, 2)) # artifically adds bars of the length of the average bar length in the song
     downbeats_times.append(song_length) # adding the last downbeat
-    
     return frontiers_to_segments(downbeats_times)
+    
+def get_beats_from_audio_msaf(signal, sr, hop_length):
+    _, audio_percussive = librosa.effects.hpss(signal)
+    
+    # Compute beats
+    _, beat_frames = librosa.beat.beat_track(y=audio_percussive, sr=sr, hop_length=hop_length)
+
+    # To times
+    beat_times = librosa.frames_to_time(beat_frames, sr=sr,hop_length=hop_length)
+
+    # TODO: Is this really necessary?
+    if len(beat_times) > 0 and beat_times[0] == 0:
+        beat_times = beat_times[1:]
+        beat_frames = beat_frames[1:]
+        
+    return beat_times, beat_frames
+    
+# %% Read and treat inputs
+def get_beats_from_audio_madmom(song_path):
+    """
+    TODO
+    """
+    act = bt.TCNBeatProcessor()(song_path)
+    proc = bt.BeatTrackingProcessor(fps=100)
+    song_beats = proc(act)
+    
+    # beats_times = []    
+    # if song_beats[0][1] != 1: # Adding a first downbeat at the start of the song
+        # beats_times.append(0.1)
+    # for beat in song_beats:
+        # if beat[1] == 1: # If the beat is a downbeat
+            # downbeats_times.append(beat[0])
+            
+    return frontiers_to_segments(list(song_beats))
 
 def get_segmentation_from_txt(path, annotations_type):
     """
@@ -399,7 +427,39 @@ def align_frontiers_on_bars(frontiers, bars):
             else:
                 frontiers_on_bars.append(bars[i][0])
     return frontiers_on_bars
-            
+
+def get_median_hop(bars, subdivision = 96, sampling_rate = 44100):
+    """
+    Returns the median hop length in the song, used for audio reconstruction.
+    The rationale is that all bars are sampled with 'subdivision' number of frames, 
+    but they can be of different lengths in absolute time.
+    Hence, the time gap between two consecutive frames (the hop length) can differ between bars.
+    For reconstruction, we use the median hop length among all bars.
+
+    Parameters
+    ----------
+    bars : list of tuples of float
+        The bars, as (start time, end time) tuples.
+    subdivision : integer, optional
+        The number of subdivision of the bar to be contained in each slice of the tensor.
+        The default is 96.
+    sampling_rate : integer, optional
+        The sampling rate of the signal, in Hz.
+        The default is 44100.
+
+    Returns
+    -------
+    integer
+        The median hop length in these bars.
+
+    """
+    hops = []
+    for bar_idx in range(1, len(bars)):
+        len_sig = bars[bar_idx][1] - bars[bar_idx][0]
+        hop = int(len_sig/subdivision * sampling_rate)
+        hops.append(hop)
+    return int(np.median(hops)) # Generally used for audio reconstruction
+
 # %% Sonification of the segmentation
 def sonify_frontiers_path(audio_file_path, frontiers_in_seconds, output_path):
     """
@@ -420,7 +480,7 @@ def sonify_frontiers_path(audio_file_path, frontiers_in_seconds, output_path):
     Nothing, but writes a song at output_path
 
     """
-    the_signal, sampling_rate = sf.read(audio_file_path)
+    the_signal, sampling_rate = librosa.load(audio_file_path, sr=None)
     sonify_frontiers_song(the_signal, sampling_rate, frontiers_in_seconds, output_path)
 
 def sonify_frontiers_song(song_signal, sampling_rate, frontiers_in_seconds, output_path):
@@ -523,14 +583,14 @@ def compute_score_of_segmentation(reference, segments_in_time, window_length = 0
     ref_intervals, useless = mir_eval.util.adjust_intervals(reference,t_min=0)
     est_intervals, useless = mir_eval.util.adjust_intervals(np.array(segments_in_time), t_min=0, t_max=ref_intervals[-1, 1])
     try:
-        return mir_eval.segment.detection(ref_intervals, est_intervals, window = window_length, trim = False)
+        return mir_eval.segment.detection(ref_intervals, est_intervals, window = window_length, trim = True)
     except ValueError:
         cleaned_intervals = []
         #print("A segment is (probably) composed of the same start and end. Can happen with time -> bar -> time conversion, but should'nt happen for data originally segmented in bars.")
         for idx in range(len(est_intervals)):
             if est_intervals[idx][0] != est_intervals[idx][1]:
                 cleaned_intervals.append(est_intervals[idx])
-        return mir_eval.segment.detection(ref_intervals, np.array(cleaned_intervals), window = window_length, trim = False)
+        return mir_eval.segment.detection(ref_intervals, np.array(cleaned_intervals), window = window_length, trim = True)
 
 def compute_median_deviation_of_segmentation(reference, segments_in_time):
     """
@@ -563,9 +623,7 @@ def compute_median_deviation_of_segmentation(reference, segments_in_time):
 def compute_rates_of_segmentation(reference, segments_in_time, window_length = 0.5):
     """
     Computes True Positives, False Positives and False Negatives from estimated segments and the reference, both in seconds.    
-    Scores are computed from the mir_eval toolbox.
-    (What happens is that precision/rap/F1 are computed via mir_eval, by computing these rates but never releasing them.
-    Hence, they are recomputed here from these values.)
+    Scores are computed from the mir_eval toolbox. (In fact, the code is extracted from mir_eval.segment.detection)
 
     Parameters
     ----------
@@ -589,12 +647,85 @@ def compute_rates_of_segmentation(reference, segments_in_time, window_length = 0
         The number of False Negatives,
         ie the number of frontiers undetected (accurate frontiers which are not found in teh estimation).
 
-    """
-    ref_intervals, useless = mir_eval.util.adjust_intervals(reference,t_min=0)
-    prec, rec, _ = compute_score_of_segmentation(reference, segments_in_time, window_length = window_length)
-    tp = int(round(rec * (len(ref_intervals) + 1)))
-    fp = int(round((tp * (1 - prec))/prec))
-    fn = int(round((tp * (1 - rec))/rec))
+    """  
+    reference_intervals, _ = mir_eval.util.adjust_intervals(reference,t_min=0)
+    estimated_intervals, _ = mir_eval.util.adjust_intervals(segments_in_time, t_min=0, t_max=reference_intervals[-1, 1])
+    
+    mir_eval.segment.validate_boundary(reference_intervals, estimated_intervals, False)
+
+    # Convert intervals to boundaries
+    reference_boundaries = mir_eval.util.intervals_to_boundaries(reference_intervals)
+    estimated_boundaries = mir_eval.util.intervals_to_boundaries(estimated_intervals)
+
+    # If we have no boundaries, we get no score.
+    if len(reference_boundaries) == 0 or len(estimated_boundaries) == 0:
+        return 0, 0, 0
+
+    tp = len(mir_eval.util.match_events(reference_boundaries,estimated_boundaries,window_length))
+    fp = len(estimated_boundaries) - tp
+    fn = len(reference_boundaries) - tp
+    
     return tp, fp, fn
         
 
+# %% High level encapsulation of the computation of scores, based on segments.
+## Tolerances are MIREX standards in time (0.5s and 3s), or 0 and 1 bar when barwise aligned.
+def get_scores_from_segments_in_time(segments_in_time, ref_tab):
+    if type(ref_tab[0][0]) != np.ndarray: # ref_tab consist in the references, and should be nested in an array (for consistency).
+        ref_tab = [ref_tab]
+
+    res = -math.inf * np.ones((2, 3))
+
+    prec05, rap05, f_mes05 = compute_score_of_segmentation(ref_tab[0], segments_in_time, window_length = 0.5)
+    prec3, rap3, f_mes3 = compute_score_of_segmentation(ref_tab[0], segments_in_time, window_length = 3)
+    res = [[round(prec05,4),round(rap05,4),round(f_mes05,4)], [round(prec3,4),round(rap3,4),round(f_mes3,4)]]
+    
+    if len(ref_tab) > 1:
+        nd_prec05, nd_rap05, nd_f_mes05 = compute_score_of_segmentation(ref_tab[1], segments_in_time, window_length = 0.5)
+        nd_prec3, nd_rap3, nd_f_mes3 = compute_score_of_segmentation(ref_tab[1], segments_in_time, window_length = 3)
+        if nd_f_mes05 + nd_f_mes3 > f_mes05 + f_mes3:
+            res = [[round(nd_prec05,4),round(nd_rap05,4),round(nd_f_mes05,4)], [round(nd_prec3,4),round(nd_rap3,4),round(nd_f_mes3,4)]]
+    
+    return res
+
+def get_scores_in_time_from_barwise_segments(segments, bars, ref_tab):
+    segments_in_time = segments_from_bar_to_time(segments, bars)
+    return get_scores_from_segments_in_time(segments_in_time, ref_tab)
+    
+def get_scores_in_bars_from_barwise_segments(segments, bars, ref_tab):
+    res = -math.inf * np.ones((2, 3))
+
+    if type(ref_tab[0][0]) != np.ndarray: # ref_tab consist in the references, and should be nested in an array (for consistency between double anntoations in SALAMI and single in RWC).
+        ref_tab = [ref_tab]
+
+    ref0_in_bars = np.array(segments_from_time_to_bar(ref_tab[0], bars))
+    
+    prec0bar, rap0bar, f_mes0bar = compute_score_of_segmentation(ref0_in_bars, segments, window_length = 0)
+    prec1bar, rap1bar, f_mes1bar = compute_score_of_segmentation(ref0_in_bars, segments, window_length = 1)
+    res = [[round(prec0bar,4),round(rap0bar,4),round(f_mes0bar,4)], [round(prec1bar,4),round(rap1bar,4),round(f_mes1bar,4)]]
+
+    if len(ref_tab) > 1:
+        ref1_in_bars = np.array(segments_from_time_to_bar(ref_tab[1], bars))
+
+        nd_prec0bar, nd_rap0bar, nd_f_mes0bar = compute_score_of_segmentation(ref1_in_bars, segments, window_length = 0)
+        nd_prec1bar, nd_rap1bar, nd_f_mes1bar = compute_score_of_segmentation(ref1_in_bars, segments, window_length = 1)
+        if nd_f_mes0bar + nd_f_mes1bar > f_mes0bar + f_mes1bar:
+            res = [[round(nd_prec0bar,4),round(nd_rap0bar,4),round(nd_f_mes0bar,4)], [round(nd_prec1bar,4),round(nd_rap1bar,4),round(nd_f_mes1bar,4)]]
+    
+    return res
+                                          
+def get_scores_switch_time_alignment(time_alignment, segments, bars, ref_tab):
+    if type(ref_tab[0][0]) != np.ndarray: # ref_tab consist in the references, and should be nested in an array (for consistency).
+        ref_tab = [ref_tab]
+
+    # Tolerance in absolute time
+    if time_alignment in ["s", "second", "seconds"]:
+        return get_scores_in_time_from_barwise_segments(segments, bars, ref_tab)
+
+    # Tolerance barwise aligned
+    elif time_alignment in ["b", "bars", "bar", "barwise"]:
+        return get_scores_in_bars_from_barwise_segments(segments, bars, ref_tab)
+            
+    else:
+        raise NotImplementedError(f"Time alignment parameter {time_alignment} not understood")
+    
